<?php // $Id: time.admin.inc,v 1.3 2009/02/19 05:36:09 vauxia Exp $

function time_admin_overview() {
  $default_profile = _time_default_profile();

  $hdrs = array(
    array('data' => t('Title'), 'field' => 'Title'),
    array('data' => t('Handler'), 'field' => 'handler'),
    array('data' => t('Status'), 'field' => 'Status'),
    array('data' => t('Total hours'), 'field' => 'hours'),
    array(),
  );

  $sql = 'SELECT p.*, SUM(t.duration) AS hours 
    FROM {time_profile} p
    LEFT JOIN {time_entry} t USING ( tpid ) GROUP BY p.tpid';

  $res = pager_query($sql. tablesort_sql($hdrs), 100, 0, NULL);
  $rows = array();
  $profiles = array();
  while ($row = db_fetch_object($res)) {
    $profile = time_load($row);
    $rows[] = array(
      l($profile->title(), 'time/'. $row->tpid),
      $profile->handler(),
      $row->status ? t('active') : t('disabled'),
      round($row->hours),
      l(t('edit'), $_GET['q'] .'/'. $row->tpid .'/edit', array('query' => 'destination='. $_GET['q'])),
    );
    if ($row->status) {
     // $profile = time_profile_load($row->tpid);
      $profiles[(string)$row->tpid] = $row->title;
    }
  }
  $output = drupal_get_form('time_admin_global_form', $profiles);
  if ($rows) $output .= theme('table', $hdrs, $rows);
  return $output;
}

function time_admin_global_form($form_state, $profiles) {
  $form = array('#theme' => 'time_admin_global_form' );

  if (count($profiles) == 1) {
    $form['time_default_profile'] = array('#type' => 'value', '#value' => key($profiles));
  }
  else {
    $form['time_default_profile'] = array(
      '#type' => 'select',
      '#title' => t('Default time tracking profile'),
      '#default_value' => variable_get('time_default_profile', 0),
      '#options' => $profiles,
    );
    // Add a default option for the node type settings.
    $profiles = array('' => t('Use global default')) + $profiles;
  }

  $form['node_types'] = array('#type' => 'markup', '#value' => '<h3>' .t('Content type settings') .'</h3>');

  foreach (node_get_types() as $type => $info) {
    $form['time_enabled_'. $type] = array(
      '#type' => 'checkbox',
      '#title' => $info->name,
      '#default_value' => variable_get('time_enabled_'. $type, 0),
    );
    if (count($profiles) > 1) {
      $form['time_default_profile_'. $type] = array(
        '#type' => 'select',
        '#default_value' => variable_get('time_default_profile_'. $type, ''),
        '#options' => $profiles,
      );
    }
    $form['time_comment_'. $type] = array(
      '#type' => 'checkbox',
      '#title' => t('Show on comment form'),
      '#default_value' => variable_get('time_comment_'. $type, 0),
    );
  }
  return system_settings_form($form);
}

function time_admin_form($form_state, $profile = NULL) {
  $form = array('#tree' => TRUE);
  $title = t('Add a time tracking profile');
  if ($profile) $title = t('Edit @title', array('@title' => $profile->title()));
  drupal_set_title($title);

  if ($profile) {
    $handler = get_class($profile);
  }
  elseif (isset($form_state['storage']['set_handler'])) {
    foreach ($form_state['storage']['set_handler'] as $handler => $status) {
      if ($status) break;
    }
  }
  else {
    $info = module_invoke_all('time_profile_info');
    if (count($info) == 1) {
      $handler = key($info);
    }
  }

  if (!$handler) {
    // If there's  more than one backend available, provide a selection.
    $form['set_handler'] = array(
      '#type' => 'radios',
      '#title' => t('Handler'),
    );
    foreach ($info as $handler => $handler_info) {
      $form['set_handler'][$handler] = array(
        '#type' => 'radio',
        '#title' => t($handler_info['title']),
        '#description' => $handler_info['description'],
        '#return_value' => $handler,
      );
    }
  }
  else {
    if (!$profile) $profile = time_profile_load(NULL, NULL, $handler);
    $form['tpid'] = array('#type' => 'value', '#value' => $profile->tpid);
    $form['handler'] = array('#type' => 'value', '#value' => $handler);

    $form['title'] = array(
      '#type' => 'textfield',
      '#title' => t('Title'),
      '#default_value' => $profile->title(),
    );

    $form['status'] = array(
      '#type' => 'checkbox',
      '#title' => t('This profile is active'),
      '#default_value' => isset($profile->status) ? $profile->status : 1,
      '#description' => t(''),
    );
    $form['handler_options'] = $profile->options_form();
  }
  $form['submit'] = array('#type' => 'submit', '#value' => t('Save'));

  //$form['entries'] = array('#type' => 'markup', '#value' => time_overview());

  return $form;
}

function time_admin_form_validate($form, &$form_state) {
  $values = $form_state['values'];
  if (isset($values['handler'])) {
    $profile = time_profile_load($values['tpid'], NULL, $values['handler']);
    $profile->options_validate($values['handler_options']);
  }
}

function time_admin_form_submit($form, &$form_state) {
  $values = $form_state['values'];

  if (isset($values['handler']) && $values['handler']) {
    $profile = time_profile_load($values['tpid'], NULL, $values['handler']);
    $profile->options_validate($values['handler_options']);
    $profile->set_options($values['handler_options']);
    $profile->set_title($values['title']);
    $profile->set_status($values['status']);
    $profile->save_profile($values['handler_options']);

    unset($form_state['rebuild'], $form_state['storage']);
    $form_state['redirect'] = 'admin/settings/time';
  }
  elseif ($values['set_handler']) {
    if (!count($form_state['storage'])) {
      $form_state['storage'] = $form_state['values'];
      $form_state['rebuild'] = TRUE;
    }
  }
}

/**
 * Provide a default listing of time entries.
 */
function time_overview($profile = NULL, $node = NULL, $account = NULL) {
  $hdrs = array(
    array('data' => t('Date'), 'field' => 'timestamp', 'sort' => 'desc'), 
    array('data' => t('User'), 'field' => 'uid'), 
    array('data' => t('Node'), 'field' => 'nid'), 
    array('data' => t('Activity'), 'field' => 'activity'), 
    array('data' => t('Note'), 'field' => 'note'), 
    array('data' => t('Hours'), 'field' => 'duration'), 
    array(),
  );

  $where = $args = array();
  $args[] = 'SELECT t.*, n.title as node_title 
    FROM {time_entry} t LEFT JOIN {node} n USING ( nid )';
  $args[] = 100; // Pager limit.
  $args[] = 0; // Pager id.
  $args[] = NULL;

  if (!user_access('view time entries')) {
    global $user;
    $where[] = 't.uid = %d';
    $args[] = $user->uid;
  }

  if ($profile) {
    $where[] = 't.tpid = %d';
    $args[] = $profile->tpid;
  }

  if ($account) {
    $where[] = 't.uid = %d';
    $args[] = $account->uid;
    unset($hdrs[1]); // Listing the user in the results would be redundant.
  }

  if ($node) {
    $where[] = 't.nid = %d';
    $args[] = $node->nid;
    unset($hdrs[2]); // Listing the node in the results would be redundant.
  }

  if ($where) {
    $args[0] .= ' WHERE '. join(' AND ', $where);
    //$args[0] .= ' ORDER BY timestamp DESC ';
  }

  $res = call_user_func_array('pager_query', $args);

  $rows = array();
  while ($row = db_fetch_object($res)) {
    $record  = array();
    $time_entry = time_load($row, $node);

    $record[] = $time_entry->timestamp();
    if (!$account) $record[] = theme('username', $time_entry->user());
    if (!$node) $record[] = l($row->node_title, 'node/'. $row->nid);
    $record[] = $time_entry->activity();
    $record[] = $time_entry->note();
    $record[] = l($time_entry->duration(), 'time/'. $row->tpid);
    $record[] = l(t('edit'), $_GET['q'] .'/'. $row->teid .'/edit', array('query' => 'destination='. $_GET['q']));

    $rows[] = $record;
  }

  //$output = l('Add a time entry', 'time/add');
  $output = drupal_get_form('time_form', NULL, $node);
  if ($rows) $output .= theme('table', $hdrs, $rows);
  return $output;
}
