<?php // $Id: time_entry.inc,v 1.4 2010/01/11 22:30:03 vauxia Exp $

class time_entry {
  var $tpid;
  var $title;
  var $options;
  var $status;

  var $teid;
  var $uid;
  var $nid;
  var $cid;
  var $note;
  var $timestamp;
  var $duration;
  var $activity;
  var $billable = 1;

  function __construct($row = NULL) {
    $row = (object) $row;
    foreach (array_keys((array) $this) as $var) {
      $func = 'set_'. $var;
      if (method_exists($this, $func)) {
        if ($row && isset($row->$var)) {
          $this->$func($row->$var);
        }
        else {
          $this->$func();
        }
      }
      else {
        $this->$var = $row->$var;
      }
    }
  }

  function set_status($status = NULL) {
    $this->status = (int) $status;
  }

  function set_title($title = NULL) {
    if (!is_null($title)) $this->title = $title;
  }

  function title() {
    return filter_xss($this->title);
  }

  function handler() {
    return get_class($this);
  }

  function set_options($options = NULL) {
    if (!is_null($options)) $this->options = $options;
  }

  function options($key = NULL) {
    if ($key) return $this->options[$key];
    return $this->options;
  }

  function options_form() {
    $form = array();
    $form['activities'] = array(
      '#type' => 'textarea',
      '#title' => t('Activities'),
      '#default_value' => $this->options('activities'),
      '#description' => t('A list of available activities. Enter one value per line, in the format key|label. The key is the value that will be stored in the database.  If this is left blank, no activities will be associated with time entries.')
    );

    return $form;
  }

  function options_validate($values) {
  }

  function form() {
    $user = $this->user();
    $form = array();

    $form['teid'] = array( '#type' => 'value', '#value' => $this->teid );
    $form['tpid'] = array( '#type' => 'value', '#value' => $this->tpid );
    $form['nid'] = array();
    $form['cid'] = array();
    $form['uid'] = array(
      '#title' => t('User'),
      '#access' => user_access('administer time tracking'),
      '#size' => '10',
      '#type' => 'textfield',
      '#default_value' => $user->name,
      '#autocomplete_path' => 'user/autocomplete',
    );
    $form['timestamp'] = array(
      '#title' => t('Date'),
      '#type' => 'textfield',
      '#size' => '10',
      '#default_value' => $this->timestamp(),
    );
    $form['note'] = array(
      '#title' => t('Comment'),
      '#type' => 'textfield',
      '#size' => '50',
      '#default_value' => $this->note(),
    );
    $form['duration'] = array(
      '#title' => t('Hours'),
      '#type' => 'textfield',
      '#size' => '10',
      '#default_value' => $this->duration(),
      '#required' => TRUE,
    );
    if ($activities = $this->activities()) {
      $form['activity'] = array(
        '#title' => t('Activity'),
        '#type' => 'select',
        '#default_value' => array($this->activity()),
        '#options' => $activities,
      );
    }
    $form['billable'] = array(
      '#title' => t('Billable'),
      '#type' => 'checkbox',
      '#default_value' => $this->billable,
    );
    if ($this->billed()) $form['billable']['#disabled'] = TRUE;

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save'),
    );

    return $form;
  }

  function validate() {
    // The time_entry class simply returns true. You can choose to override and
    // validate node types, user access, etc; which will prevent use.
    return TRUE;
  }

  function save($values = NULL) {
    if ($values) $this->__construct($values);
    $update = isset($this->teid) ? 'teid' : NULL;
    if ($this->duration) {
      drupal_write_record('time_entry', $this, $update);
    }
  }

  function save_profile() {
    $update = isset($this->tpid) ? 'tpid' : NULL;
    $this->handler = get_class($this);
    drupal_write_record('time_profile', $this, $update);
  }

  function activities() {
    $activities = array();
    if ($data = $this->options['activities']) {
      foreach (explode("\n", $data) as $item) {
        list($key, $val) = explode('|', $item);
        if (!$val) $val = $key;
        $activities[filter_xss($key)] = filter_xss($val);
      }
    }
    return $activities;
  }

  function set_uid($uid = NULL) {
    if (!$uid) {
      global $user;
      $uid = $user->uid;
    }
    elseif (!is_numeric($uid)) {
      $user = user_load(array('name' => $uid));
      $uid = $user->uid;
    }
    $this->uid = $uid;
  }

  function user() {
    if ($this->uid) return user_load($this->uid);
  }

  function set_nid($nid = NULL) {
    $this->nid = $nid;
  }

  function node() {
    if ($this->nid) return node_load($this->nid);
  }

  function set_cid($cid = NULL) {
    $this->cid = $cid;
  }

  function comment() {
    if ($this->cid) return comment_load($this->cid);
  }
  
  function set_note($note = NULL) {
    $this->note = filter_xss($note);
  }

  function note($format = NULL) {
    return filter_xss($this->note);
  }

  function set_timestamp($timestamp = NULL) {
    if ($timestamp) {
      if (is_numeric($timestamp)) {
        $this->timestamp = $timestamp;
      }
      else {
        $this->timestamp = strtotime($timestamp);
      }
    }
    else {
      $this->timestamp = time();
    }
  }

  function timestamp($format = NULL) {
    if (!$this->timestamp) $this->timestamp = time();
    return date('m/d/Y', $this->timestamp);
    // I'd prefer to use this, but it won't be re-parseable. :(
    // return format_date($this->timestamp, 'small');
  }

  function set_duration($duration = NULL) {
    if ($duration) $this->duration = (float) $duration;
  }

  function duration($format = NULL) {
    return $this->duration;
  }

  function set_activity($activity = NULL) {
    $this->activity = $activity;
  }

  function activity($format = 'val') {
    if ($format == 'val') {
      $activities = $this->activities();
      return $activities[$this->activity];
    }
    return $this->activity;
  }

  function set_billable($billable = 1) {
    $this->billable = (int) $billable;
  }

  function billable() {
    return (bool) ($this->billable != 0);
  }

  function set_billed($billed = TRUE) {
    if ($billed) $this->billable = -1;
  }

  function billed() {
    return (bool) ($this->billable == -1);
  }
}
