<?php // $Id: time.views.inc,v 1.2 2010/01/11 22:27:44 vauxia Exp $

/**
 * Views related functionality for the time tracking module
 *
 * @package time
 */

/**
 * Implementation of hook_views_data().
 */
function time_views_data() {
  $data = array();

  $data['time_entry']['table']['group'] = t('Time tracking');

  // Advertise this table as a possible base table.
  $data['time_entry']['table']['base'] = array(
    'field' => 'teid',
    'title' => t('Time'),
    'help' => t('Time tracking entries.'),
  );

  // Joins.
  $data['time_entry']['table']['join'] = array(
    'node' => array(
      'left_field' => 'nid',
      'field' => 'nid',
    ),
  );
  $data['time_entry']['table']['join'] = array(
    'users' => array(
      'left_field' => 'uid',
      'field' => 'uid',
    ),
  );
  $data['time_entry']['table']['join'] = array(
    'comments' => array(
      'left_field' => 'cid',
      'field' => 'cid',
    ),
  );

  // Fields.
  $data['time_entry']['timestamp'] = array(
    'title' => t('Activity date'),
    'help' => t('Date the time was entered.'),
    'field' => array(
      'handler' => 'views_handler_field_date',
      'click sortable' => TRUE,
     ),
    'sort' => array(
      'handler' => 'views_handler_sort_date',
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_date',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_date',
    ),
  );

  $data['time_entry']['activity'] = array(
    'title' => t('Activity'),
    'help' => t('Activity performed during this time entry.'),
    'field' => array(
      'handler' => 'views_handler_field',
       'click sortable' => TRUE,
     ),
     'sort' => array(
       'handler' => 'views_handler_sort',
     ),
     'filter' => array(
       'handler' => 'views_handler_filter_string',
     ),
     'argument' => array(
       'handler' => 'views_handler_argument_string',
     ),
  );

  // TODO need a custom handler
  $data['time_entry']['duration'] = array(
    'title' => t('Duration'),
    'help' => t('Time spent during a time entry.'),
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE,
     ),
    'sort' => array(
      'handler' => 'views_handler_sort_numeric',
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_numeric',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
    ),
  );

  $data['time_entry']['uid'] = array(
    'title' => t('User'),
    'help' => t('The user that entered time.'),
    'relationship' => array(
      'base' => 'users',
      'field' => 'uid',
      'handler' => 'views_handler_relationship',
      'label' => t('Time tracking user'),
    ),
  );

  $data['time_entry']['nid'] = array(
    'title' => t('Node'),
    'help' => t('The node related to the time entry.'),
    'relationship' => array(
      'base' => 'node',
      'field' => 'nid',
      'handler' => 'views_handler_relationship',
      'label' => t('Node'),
    ),
  );

  $data['time_entry']['cid'] = array(
    'title' => t('Comment'),
    'help' => t('The comment related to the time entry.'),
    'relationship' => array(
      'base' => 'comment',
      'field' => 'cid',
      'handler' => 'views_handler_relationship',
      'label' => t('Comment'),
    ),
  );

  $data['time_entry']['note'] = array(
    'title' => t('Note'),
    'help' => t('Notes or details on the time tracking activity.'),
     'field' => array(
      'handler' => 'views_handler_field',
     ),
     'sort' => array(
       'handler' => 'views_handler_sort',
     ),
  );

  $data['time_entry']['billable'] = array(
    'title' => t('Billable'),
    'help' => t('Whether or not the time entered is billable.'),
    'field' => array(
      'handler' => 'views_handler_field_boolean',
      'click sortable' => TRUE,
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_boolean_operator',
      'label' => t('Billable'),
      'type' => 'yes-no',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
  );

  return $data;
}

/**
 * Implementation of hook_views_handlers().
 */
function time_views_handlers_not() {
  return array(
    'info' => array(
      'path' => drupal_get_path('module', 'time') .'/includes/views',
    ),
  );
}
